#include <avr/io.h>
#include "CFAL12832A-022.h"

// all on PORTC
#define	LCD_CD	PC7   
#define	LCD_WR  PC6	// WR in 6800 mode
#define	LCD_RD	PC5	// RD in 6800 mode
#define	LCD_CS	PC4
#define	LCD_RES	PC2

#define CLR_CD PORTC &= ~(1<<LCD_CD);
#define SET_CD PORTC |=  (1<<LCD_CD);

#define CLR_CS PORTC &= ~(1<<LCD_CS);
#define SET_CS PORTC |=  (1<<LCD_CS);

#define CLR_RESET PORTC &= ~(1<<LCD_RES);
#define SET_RESET PORTC |=  (1<<LCD_RES);

// 6800 mode pin functions
#define CLR_WR PORTC &= ~(1 << LCD_WR);	// 6800 mode
#define SET_WR PORTC |=  (1 << LCD_WR);	// 6800 mode
#define CLR_RD PORTC &= ~(1 << LCD_RD);	// 6800 mode
#define SET_RD PORTC |=  (1 << LCD_RD);	// 6800 mode

void delay(uint32_t twait)
{
	while (twait--)
		asm volatile ("nop");
}

void sendCommand(uint8_t command)
{
    CLR_CS;
    CLR_CD;
    SET_RD;
	PORTA = command;
	CLR_WR;
	SET_WR;
    SET_CS;
}


void sendData(uint8_t data)
{
    CLR_CS;
    SET_CD;
    SET_RD;
	PORTA = data;
	CLR_WR;
	SET_WR;
    SET_CS;
}

void lcd_clr(uint8_t color)
{
	int i,p;

	sendCommand(0x40);

	for (p=0;p<8;p++) // pages
	{
		sendCommand(0xb0 + p);
		sendCommand(0x10);
		sendCommand(0x00);
		for (i=0;i<132;i++)	// ?????
		{
			sendData(color);
		}
	}
}

typedef uint8_t bitmap_t[8][128];

void bmp(bitmap_t b)
{
	int i,j;
	uint8_t X = 0xB0;
	for (j=0;j<4;j++)
	{
		sendCommand(X);
		X++;
		sendCommand(0x10);
		sendCommand(0x04);

		for (i=0;i<128;i++)
		{
			sendData( b[j][i] );
		}
		//sendData(0);
		//sendData(0);
		//sendData(0);
		//sendData(0);
	}

}

void init_display()
{
	sendCommand(0xAE);  // Display Off
	   
	sendCommand(0xAD);  // Master Configuration
	sendCommand(0x8E);  // Select external VCC supply

	sendCommand(0xA8);  // Select Multiplex Ratio
	sendCommand(0x1F); // Default =>0x1F(1/32 Duty) 0x3F (1/64 Duty)

	sendCommand(0xD3);  //Setting Display Offset
	sendCommand(0x00);  //00H Reset

	sendCommand(0x00); //Set Column Address LSB

	sendCommand(0x10); //Set Column Address MSB

	//
	sendCommand(0x40);  //Set Display Start Line

	sendCommand(0x00);  //;Set Memory Addressing Mode Default => 0x02
	//0x00 => Horizontal Addressing Mode
	sendCommand(0xA6);  //Set Normal Display
	   
	sendCommand(0xDB); //Set Deselect Vcomh level
	sendCommand(0x3C); //~0.83xVCC

	sendCommand(0xA4);  //Entire Display ON

	sendCommand(0x81);  //Set Contrast Control for Bank 0
	sendCommand(0xBF);

	sendCommand(0xD5);  //SET DISPLAY CLOCK
	sendCommand(0xF0);  //105HZ

	sendCommand(0xD8); //Select Area color ON/OFF
	sendCommand(0x05); //MONO Mode and Low Power display Mode

	sendCommand(0xA1); //Set Segment Re-Map Default => 0xA0
	//0xA1 (0x01) => Column Address 0 Mapped to SEG131
	sendCommand(0x08); //Set COM Output Scan Direction Default => 0xC0
	//0xC8 (0x08) => Scan from COM63 to 0
	sendCommand(0xDA);  //Set COM Hardware Configuration
	sendCommand(0x12); //Alternative COM Pin

	sendCommand(0xD9); //Set Pre-Charge period
	sendCommand(0xF2); //Refer to SPEC Page34
	sendCommand(0xFF); //

	sendCommand(0xAF); // Display ON
}

/*******************************************************************************************/
/*******************************************************************************************/
/*******************************************************************************************/
uint8_t inverted;
void invert_lcd()
{
	if(inverted)
	{
		sendCommand(0xa6); //set inverse display
		inverted = 0;
	}
	else
	{
		sendCommand(0xa7); //set inverse display
		inverted = 1;
	}
}
int main( void )
{
	inverted = 0;
	//PORTA = D0-D7
	PORTA = 0x00;
	DDRA = 0xFF;


	PORTD = 0;		// all off
	DDRD |= (1<<3);	// VPP output
	DDRD |= 0x06;	// led pins output

	
	PORTC = 0b11011110;
	DDRC = 0xFE;
	
	//Thump the reset
	delay(20000L);
	PORTC &= ~LCD_RES;
	delay(10000);
	PORTC |= LCD_RES;

	delay(20000);

	// enable VPP
	PORTD |= (1<<3);

	delay(200000L);

	init_display();

	bmp(CFAL12832A2);
	delay(2500000L);
	while (1)
	{
//		invert_lcd();
	}
	return 0;
}

